/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdisc.h"

#include "kdcfilefactory.h"
#include "lib/kdclib.h"

#include <QTreeWidgetItem>
#include <QTreeWidgetItemIterator>
#include <QStringList>
#include <QHeaderView>
#include <QSize>
#include <QList>
#include <QInputDialog>
#include <QPainter>
#include <QPainterPath>
#include <QRect>

#include <KLocale>

KDCDisc::KDCDisc( KDC::DiscType type, int id, QWidget *parent ) 
	: QTreeWidget( parent )
{
	p_discType = type;
	p_menu = FALSE;
	p_size = 0;
	p_drag = FALSE;
	p_movieID = 0;
	p_id = id;
	p_key = -1;

	p_settings = new KDCDiscSettings( this );
	connect( p_settings, SIGNAL( mediaSizeChanged() ), this, SIGNAL( settingsChanged() ) );
	connect( p_settings, SIGNAL( discOutputDirectoryChanged() ), this, SIGNAL( settingsChanged() ) );

	p_itemDelegate = new KDCDiscItemDelegate;
	setItemDelegate( p_itemDelegate );
	setUniformRowHeights( FALSE );
	setAcceptDrops( TRUE );
	setDropIndicatorShown( TRUE );
	setRootIsDecorated( TRUE );
	setMouseTracking( TRUE );
	setContextMenuPolicy( Qt::CustomContextMenu );
	setCursor( Qt::PointingHandCursor );
	setColumnCount( 1 );
	setIconSize( QSize( 22, 22 ) );
	setSelectionBehavior( QAbstractItemView::SelectRows );
	setVerticalScrollMode( QAbstractItemView::ScrollPerPixel );
 
	QStringList headers;
	headers.append( i18n("Disc Structure") );
	setHeaderLabels( headers );
	header()->setDefaultAlignment( Qt::AlignHCenter | Qt::AlignVCenter );

	connect( this, SIGNAL( itemDoubleClicked( QTreeWidgetItem * , int ) ), this, SLOT( on_itemDoubleClicked( QTreeWidgetItem * , int ) ) );
	connect( this, SIGNAL( currentItemChanged( QTreeWidgetItem * , QTreeWidgetItem * ) ), this, SLOT ( on_currentItemChanged( QTreeWidgetItem * , QTreeWidgetItem * ) ) );
	connect( this, SIGNAL( itemEntered( QTreeWidgetItem * , int ) ), this, SLOT( on_itemEntered( QTreeWidgetItem * , int ) ) );

	KDCFileFactory::createDirectory( KDCFileFactory::discDirectory( this ) );
	refresh();
}

KDCDisc::~KDCDisc()
{
	KDCFileFactory::removeDirectory( KDCFileFactory::discDirectory( this ) );
	delete p_itemDelegate;
	delete p_settings;
}

KDC::DiscType KDCDisc::discType() const
{
	return p_discType;
}

KDCDiscSettings* KDCDisc::settings() const
{
	return p_settings;
}

bool KDCDisc::isMenuAvalible()
{
	return p_menu;
}

int KDCDisc::id() const
{
	return p_id;
}

QTreeWidgetItem* KDCDisc::currentMovie() const
{
	return p_currentMovie;
}

KDCMovie* KDCDisc::movie( int id )
{
	KDCMovie *tmp;
	QTreeWidgetItemIterator it( this );
	while ( *it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			tmp = static_cast<KDCMovie*>( *it );
			if ( tmp->id() == id )
				return tmp;
		}
		++it;
	}
	return NULL;
}

KDCMovieGroup* KDCDisc::movieGroup( int id )
{
	KDCMovieGroup *tmp;
	QTreeWidgetItemIterator it( this );
	while ( *it )
	{
		if ( ( *it )->type() == KDC::MovieGroup )
		{
			tmp = static_cast<KDCMovieGroup*>( *it );
			if ( tmp->id() == id )
				return tmp;
		}
		++it;
	}
	return NULL;
}

QList<QTreeWidgetItem*> KDCDisc::takeMovies()
{
	qDebug( "KDCDisc::takeMoviesToProcess()::begin" );
	QList<QTreeWidgetItem*> list;
	QTreeWidgetItemIterator it( this );
	while ( *it )
	{
		if ( ( *it )->type() == KDC::Movie || ( *it )->type() == KDC::MovieGroup )
			list.append( *it );
		++it;
	}
	qDebug( "KDCDisc::takeMoviesToProcess()::end" );
	return list;
}

quint64 KDCDisc::size() const
{
	return p_size;
}

void KDCDisc::computeNewSize()
{
	qDebug( "KDCDisc::computeNewSize::begin" );
	QTreeWidgetItemIterator it( this );
	KDCMovie *movie;
	KDCMovieGroup *group;
	p_size = 0;
	while( *it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( *it );
			p_size += movie->size();
		}
		else
			if ( ( *it )->type() == KDC::MovieGroup )
			{
				group = static_cast<KDCMovieGroup*>( *it );
				p_size += group->size();
			}
		++it;
	}
	qDebug( "KDCDisc::computeNewSize()::emit sizeChanged()" );
	emit settingsChanged();
	qDebug( "KDCDisc::computeNewSize()::end" );
}

void KDCDisc::on_itemDoubleClicked( QTreeWidgetItem *item, int column )
{
	qDebug( "KDCDisc::itemDoubleClicked( %d )::begin", KDCLib::isNull( item ) );
	if ( item->type() == KDC::Movie || item->type() == KDC::MovieGroup || item->type() == KDC::MovieInfo )
	{
		bool expandItem = item->isExpanded();
		if ( item->type() == KDC::MovieInfo )
			item = item->parent();
		QString name;
		KDCMovie *movie = NULL;
		KDCMovieGroup *group = NULL;
		if ( item->type() == KDC::Movie )
		{
			movie = static_cast<KDCMovie*>( item );
			name = movie->outputName();
		}
		else
		{
			group = static_cast<KDCMovieGroup*>( item );
			name = group->outputName();
		}
		bool ok;
		QString text = QInputDialog::getText( this, QString( "KDVDCreator" ), i18n( "Set output name" ),
																					QLineEdit::Normal, name, &ok );
		if ( ok && !text.isEmpty() )
		{
			if ( movie )
				movie->setOutputName( text );
			else
				group->setOutputName( text );
		}
		item->setExpanded( !expandItem );
	}
	qDebug( "KDCDisc::itemDoubleClicked()::end" );
}

void KDCDisc::on_currentItemChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous )
{
	qDebug( "KDCDisc::currentItemChanged( %d )::begin", KDCLib::isNull( current ) );
	setUpdatesEnabled( FALSE );
	if ( current )
	{
		qDebug( "KDCDisc::currentItemChanged()::current,type = %d", current->type() );
		if ( p_key == Qt::Key_Down )
		{
			if ( current->type() == KDC::MovieInfo && itemBelow( current ) )
				current = itemBelow( current );
			p_key = -1;
		}
		if ( current->type() == KDC::MovieInfo )
			current = current->parent();
		if ( current->type() == KDC::Movie || current->type() == KDC::MovieGroup )
		{
			if ( current != p_currentMovie )
				p_currentMovie = current;
			if ( p_currentMovie->type() == KDC::MovieGroup )
			{
				KDCMovieGroup *group = static_cast<KDCMovieGroup*>( p_currentMovie );
				if ( group->count() < 2 )
					p_currentMovie = NULL;
			}
		}
		else
			p_currentMovie = NULL;

		qDebug( "KDCDisc::currentItemChanged()::object, type = %d", current->type() );
		if ( current->type() == KDC::MovieGroup && !p_currentMovie )
			p_currentObject = NULL;
		else
			//if ( tmp != current || ( tmp->type() == KDC::MovieGroup && current->type() == KDC::MovieGroup ) )
			{
				p_currentObject = current;
				setCurrentItem( p_currentObject );
			}
	}
	else
	{
		p_currentMovie = NULL;
		p_currentObject = NULL;
	}
	emit objectChanged( p_currentObject );
	setUpdatesEnabled( TRUE );
	qDebug( "KDCDisc::currentItemChanged()::end::object = %d, movie = %d", KDCLib::isNull( p_currentObject ), KDCLib::isNull( p_currentMovie ) );
	if ( p_currentObject )
		qDebug( "KDCDisc::currentItemChanged()::object, type = %d, item, type = %d", p_currentObject->type(), currentItem()->type() );
}

void KDCDisc::drawRow( QPainter *painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
	painter->save();
	QTreeWidgetItem *item = itemFromIndex( index );
	p_itemDelegate->setDrawItem( NULL );
	QBrush brush = option.palette.highlight();
	QColor color = brush.color();
	color.setAlpha( 50 );
	brush.setColor( color );
	QRect rect = option.rect;
	rect.setX( 0 );

	if ( item == p_objectMouse )
		painter->fillRect( rect, brush );
	if ( p_movieMouse )
	{
		if ( item == p_movieMouse->child( 0 ) )
			painter->fillRect( rect, brush );
	}

	if ( item == p_currentObject )
	{
		p_itemDelegate->setDrawItem( item );
		painter->fillRect( rect, option.palette.highlight() );
	}
	if ( p_currentMovie )
	{
		if ( item == p_currentMovie->child( 0 ) )
		{
			p_itemDelegate->setDrawItem( item );
			painter->fillRect( rect, option.palette.highlight() );
		}
	}

	painter->restore();
	QTreeWidget::drawRow( painter, option, index );
}

void KDCDisc::drawBranches( QPainter *painter, const QRect& rect, const QModelIndex& index ) const
{

}

int KDCDisc::maxSubtitleSreams() const
{
	return p_max_subtitleStreams;
}

int KDCDisc::maxAudioStreams() const
{
	return p_max_audioStreams;
}

QStringList KDCDisc::mimeTypes() const
{
	QStringList qstrList;
	qstrList.append("text/uri-list");
	return qstrList;
}

Qt::DropActions KDCDisc::supportedDropActions() const
{
	return Qt::MoveAction | Qt::CopyAction;
}

QStringList KDCDisc::codeMovies( KDCMovie *movie )
{
	qDebug( "KDCDVDDisc::codeMovies()::begin" );
	QStringList files;

	KDCMovie *tmp;
	QTreeWidgetItemIterator it( this );
	while( *it )
	{
		if ( ( *it )->type() == KDC::Movie )
		{
			tmp = static_cast<KDCMovie*>( *it );
			if ( tmp->id() != movie->id() )
			{
				if ( tmp->audioStreamGroup() != NULL && movie->audioStreamGroup() != NULL )
				{
					if ( tmp->audioStreamGroup()->count() == movie->audioStreamGroup()->count() )
						files.append( QString( tmp->outputName() + "\t%1" ).arg( tmp->id() ) );
				}
				else
					if ( tmp->audioStreamGroup() == NULL && movie->audioStreamGroup() == NULL )
						files.append( QString( tmp->outputName() + "\t%1" ).arg( tmp->id() ) );
			}
		}
		++it;
	}
	qDebug( "KDCDVDDisc::codeMovies()::end" );
	return files;
}

QStringList KDCDisc::codeMovieGroups( KDCMovie *movie )
{
	qDebug( "KDCDVDDisc::codeMovieGroups()::begin" );
	QStringList files;

	KDCMovieGroup *group;
	QTreeWidgetItemIterator it( this );
	while ( *it )
	{
		if ( ( *it )->type() == KDC::MovieGroup )
		{
			group = static_cast<KDCMovieGroup*>( *it );
			if ( group->movie( 0 )->audioStreamGroup() != NULL && movie->audioStreamGroup() != NULL )
			{
				if ( group->movie( 0 )->audioStreamGroup()->count() == movie->audioStreamGroup()->count() )
					files.append( QString( group->outputName() + "\t%1" ).arg( group->id() ) );
			}
			else
				if ( group->movie( 0 )->audioStreamGroup() == NULL && movie->audioStreamGroup() == NULL )
					files.append( QString( group->outputName() + "\t%1" ).arg( group->id() ) );
		}
		++it;
	}
	qDebug( "KDCDVDDisc::codeMovieGroups()::end" );
	return files;
}

void KDCDisc::keyPressEvent( QKeyEvent *event )
{
	p_key = event->key();
	QTreeWidget::keyPressEvent( event );
}

void KDCDisc::on_itemEntered( QTreeWidgetItem *item, int column )
{
	setUpdatesEnabled( FALSE );
	if ( item )
	{
		if ( item != p_objectMouse )
		{
			if ( item->type() == KDC::MovieInfo )
				item = item->parent();
			if ( item->type() == KDC::Movie || item->type() == KDC::MovieGroup )
				p_movieMouse = item;
			else
				p_movieMouse = NULL;
			p_objectMouse = item;
		}
	}
	else
	{
		p_movieMouse = NULL;
		p_objectMouse = NULL;
	}
	if ( p_objectMouse == p_currentObject )
		p_objectMouse = NULL;
	if ( p_movieMouse == p_currentMovie )
		p_movieMouse = NULL;
	setUpdatesEnabled( TRUE );
}

void KDCDisc::mouseMoveEvent( QMouseEvent *event )
{
	QTreeWidgetItem *item = itemAt( event->pos() );
	if ( event->y() < 0 )
		item = NULL;
	if ( !item && p_objectMouse )
		on_itemEntered( item, 0 );
	QTreeWidget::mouseMoveEvent( event );
}

void KDCDisc::refresh()
{
	setUpdatesEnabled( FALSE );
	p_currentMovie = NULL;
	p_currentObject = NULL;
	p_movieMouse = NULL;
	p_objectMouse = NULL;
	computeNewSize();
	setUpdatesEnabled( TRUE );
	on_currentItemChanged( currentItem(), currentItem() );
}
