/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcfilefactory.h"

#include "kdcsubtitlestreamgroup.h"

#include <QDir>
#include <QFileInfo>
#include <QFile>
#include <QTextStream>

#include <KStandardDirs>
#include <kio/netaccess.h>
#include <kio/job.h>
#include <KTempDir>
#include <KUrl>
#include <kmountpoint.h>

QString KDCFileFactory::p_appsTempPath = QString();
QString KDCFileFactory::p_appsLogFileName = QString();

const QString KDCFileFactory::appTempPath()
{
	return p_appsTempPath;
}

void KDCFileFactory::setInitializeInfo( const QString& appsPath )
{
	p_appsTempPath = appsPath;
	if ( p_appsTempPath.endsWith( '/' ) )
		p_appsTempPath.resize( p_appsTempPath.size() - 1 );
	p_appsLogFileName = p_appsTempPath + "/kdclog.log";
}

const QString KDCFileFactory::logFileName()
{
	return p_appsLogFileName;
}

void KDCFileFactory::addLogInfo( const char *msg )
{
	QFile logFile( p_appsLogFileName );
	logFile.open( QIODevice::WriteOnly | QIODevice::Append );
	QTextStream st( &logFile );
	st << QString( msg ) + "\n";
	logFile.close();
}

void KDCFileFactory::clean()
{
	removeDirectory( appTempPath() );
}

KIcon KDCFileFactory::appIcon()
{
	return KIcon("media-optical");
}

const QString KDCFileFactory::discDirectory( const KDCDisc *disc )
{
	return QString( p_appsTempPath + "/disc_%1" ).arg( disc->id() );
}

const QString KDCFileFactory::discOutputDirectory( const KDCDisc *disc )
{
	return disc->settings()->outputPath() + QString( "/disc_%1" ).arg( disc->id() );
}

const QString KDCFileFactory::discConfigFile( const KDCDisc *disc )
{
	return QString( discDirectory( disc ) + "/discconfigfile.xml" );
}

const QString KDCFileFactory::movieName( const KDCMovie *movie )
{
	KDCDisc *disc = movie->disc();
	return QString( discOutputDirectory( disc ) + "/movie_" + QString::number( disc->id() ) + "_%1.mpg" ).arg( movie->id() );
}

const QString KDCFileFactory::movieFrame( const KDCMovie *movie )
{
	return QString( discDirectory( movie->disc() ) + "/frame_%1.png" ).arg( movie->id() );
}

const QString KDCFileFactory::movieSubtitleFile( const KDCMovie *movie, int id )
{
	return QString( discDirectory( movie->disc() ) + "/sub_%1_%2.txt" ).arg( movie->id() ).arg( id );
}

const QString KDCFileFactory::movieSubtitleConfigFile( const KDCMovie *movie, int id )
{
	return QString( discDirectory( movie->disc() ) + "/sub_%1_%2.xml" ).arg( movie->id() ).arg( id );
}

const QString KDCFileFactory::movieDir( const KDCMovie *movie )
{
	return QString( discDirectory( movie->disc() ) + "/movie_%1" ).arg( movie->id() );
}

const QString KDCFileFactory::movieName( const KDCMovieGroup *movie )
{
	KDCDisc *disc = movie->disc();
	return QString( discOutputDirectory( disc ) + "/movie_" + QString::number( disc->id() ) + "_%1.mpg" ).arg( movie->id() );
}

const QString KDCFileFactory::movieFrame( const KDCMovieGroup *movie )
{
	return QString( discDirectory( movie->disc() ) + "/frame_%1.png" ).arg( movie->id() );
}

const QString KDCFileFactory::movieSubtitleFile( const KDCMovieGroup *movie, int id )
{
	return QString( discDirectory( movie->disc() ) + "/sub_%1_%2.txt" ).arg( movie->id() ).arg( id );
}

const QString KDCFileFactory::movieSubtitleConfigFile( const KDCMovieGroup *movie, int id )
{
	return QString( discDirectory( movie->disc() ) + "/sub_%1_%2.xml" ).arg( movie->id() ).arg( id );
}

const QString KDCFileFactory::movieDir( const KDCMovieGroup *movie )
{
	return QString( discDirectory( movie->disc() ) + "/movie_%1" ).arg( movie->id() );
}

const QString KDCFileFactory::fileName( const QString& path )
{
	QFileInfo info( path );
	return info.fileName();
}

const QString KDCFileFactory::baseFileName( const QString& path )
{
	QFileInfo info( path );
	return info.completeBaseName();
}

const QString KDCFileFactory::filePath( const QString& path )
{
	QFileInfo info( path );
	return info.absolutePath();
}

const QString KDCFileFactory::fileNameWithoutExtension( const QString& path )
{
	QFileInfo info( path );
	QString str;
	str.append( info.absolutePath() );
	str.append( "/" );
	str.append( info.completeBaseName() );
	return str;
}

QString KDCFileFactory::readAll( const QString& file )
{
	QFile in( file );
	in.open( QIODevice::ReadOnly );
	QTextStream st( &in );
	QString out = st.readAll();
	in.close();
	return out;
}

const QString KDCFileFactory::mountPoint( const QString& path )
{
	KMountPoint::List mountList = KMountPoint::currentMountPoints();
	KMountPoint *ptr = mountList.findByPath( path ).data();

	if ( ptr )
		return ptr->mountPoint();
	else
		return "/";
}

bool KDCFileFactory::createDirectory( const QString& path )
{
	QDir dir( path );
	if ( !dir.exists() )
		return KIO::NetAccess::mkdir( KUrl( path ), NULL );
	else
		return TRUE;
}

void KDCFileFactory::removeDirectory( const QString& path )
{
	QDir dir( path );
	if ( dir.exists() )
		KIO::NetAccess::del( KUrl( path ), NULL );
}

bool KDCFileFactory::removeFile( const QString& path )
{
	if ( QFile::exists( path ) )
		return QFile::remove( path );
	else
		return FALSE;
}

bool KDCFileFactory::renameFile( const QString& oldName , const QString& newName )
{
	if ( QFile::exists( oldName ) && !QFile::exists( newName ) )
		return QFile::rename( oldName, newName );
	else
		return FALSE;
}
